@@ -25,9 +25,13 @@ def compute_chessboard_corners(W, H,
                                *,
                                globs_per_camera  = ('*',),
                                corners_cache_vnl = None,
+                               image_path_prefix = None,
+                               image_directory   = None,
+
                                jobs              = 1,
                                exclude_images    = set(),
-                               weight_column_kind= 'level'):
+                               weight_column_kind= 'level',
+                               Npoint_observations_per_board_min = None):
     r'''Compute the chessboard observations and returns them in a usable form
 
 SYNOPSIS
@@ -100,6 +104,39 @@ ARGUMENTS
   corners with subsequent calls. This exists to save time where re-analyzing the
   same data multiple times.
 
+  When the corners_cache_vnl exists, and refers images that have existed on
+  disk, some logic is needed to adjust the paths in corners_cache_vnl. Because
+  those paths don't need to exist anymore and because the corners_cache_vnl file
+  and/or the images may have moved since they were detected. The logic is:
+
+    if image_path_prefix is given: we use this as a prefix to the paths in
+      corners_cache_vnl
+    elif image_directory is given: we strip out the directory from the paths in
+      corners_cache_vnl, and use image_directory instead
+    elif we're reading from a corners_cache_vnl file on disk and
+         the path in corners_cache_vnl is relative and
+         this path does not exist relative to the current directory:
+      use the directory of the corners_cache_vnl as the prefix
+    else:
+      use the path in corners_cache_vnl as is
+
+  The above logic is complex to cover the various possible case. The canonical
+  case is to put the corners.vnl file into the same directory as the images, to
+  put the filenames only into the corners.vnl, and to not have any images in the
+  current directory. If you want to be extra sure about what it's doing, pass
+  image_path_prefix or image_directory
+
+- image_path_prefix: optional argument, defaulting to None, exclusive with
+  "image_directory". Used to locate the images when reading from a given
+  corners_cache_vnl. If given, the image paths in the "paths" argument are
+  prefixed with the given string. See docs for corners_cache_vnl above
+
+- image_directory: optional argument, defaulting to None, exclusive with
+  "image_path_prefix". Used to locate the images when reading from a given
+  corners_cache_vnl. If given, we extract the filename from the image path in
+  the "paths" argument, and look for the images in the directory given here
+  instead. See docs for corners_cache_vnl above
+
 - jobs: a GNU-Make style parallelization flag. Indicates how many parallel
   processes should be invoked when computing the corners. If given, a numerical
   argument is required. If jobs<0: the corners_cache_vnl MUST already contain
@@ -120,6 +157,10 @@ ARGUMENTS
     ignored: we set output weight = -1
   - None: the 4th column should be ignored, and I set the output weight to 1.0
 
+- Npoint_observations_per_board_min: optional integer, defaulting to 2*min(W,H).
+  We throw out any images where a chessboard detection has fewer than this many
+  points.
+
 RETURNED VALUES
 
 This function returns a tuple (observations, indices_frame_camera, files_sorted)
@@ -147,6 +188,10 @@ which mrcal.optimize() expects
             weight_column_kind is None):
         raise Exception(f"weight_column_kind must be one of ('level','weight',None); got '{weight_column_kind}'")
 
+    if Npoint_observations_per_board_min is None:
+        Npoint_observations_per_board_min = 2*min(W,H)
+
+
     import os
     import fnmatch
     import subprocess
@@ -193,21 +238,17 @@ which mrcal.optimize() expects
         for i in range(Ncameras):
             files_per_camera.append([])
 
-        # images in corners_cache_vnl have paths relative to where the
-        # corners_cache_vnl lives
         corners_dir = None
 
         fd_already_opened = isinstance(corners_cache_vnl, io.IOBase)
 
-        if corners_cache_vnl is not None and not fd_already_opened:
-            corners_dir = os.path.dirname( corners_cache_vnl )
-
         pipe_corners_write_fd          = None
         pipe_corners_write_tmpfilename = None
-        if corners_cache_vnl is not None and \
-           not fd_already_opened         and \
-           os.path.isdir(corners_cache_vnl):
-            raise Exception(f"Given cache path '{corners_cache_vnl}' is a directory. Must be a file or must not exist")
+
+        if corners_cache_vnl is not None and not fd_already_opened:
+            if os.path.isdir(corners_cache_vnl):
+                raise Exception(f"Given cache path '{corners_cache_vnl}' is a directory. Must be a file or must not exist")
+            corners_dir = os.path.dirname( corners_cache_vnl )
 
         if corners_cache_vnl is not None and \
            ( fd_already_opened or \
@@ -283,19 +324,19 @@ which mrcal.optimize() expects
                     raise Exception("File '{}' expected to have {} points, but got {}". \
                                     format(context['f'], W*H, context['igrid']))
                 if context['f'] not in exclude_images and \
-                   context['Nvalidpoints'] > 3:
-                    # There is a bit of ambiguity here. The image path stored in
-                    # the 'corners_cache_vnl' file is relative to what? It could be
-                    # relative to the directory the corners_cache_vnl lives in, or it
-                    # could be relative to the current directory. The image
-                    # doesn't necessarily need to exist. I implement a few
-                    # heuristics to figure this out
-                    if corners_dir is None          or \
-                       context['f'][0] == '/'       or \
-                       os.path.exists(context['f']):
-                        filename_canonical = os.path.normpath(context['f'])
+                   context['Nvalidpoints'] > Npoint_observations_per_board_min:
+
+                    if image_path_prefix is not None:
+                        filename_canonical = f"{image_path_prefix}/{context['f']}"
+                    elif image_directory is not None:
+                        filename_canonical = f"{image_directory}/{os.path.basename(context['f'])}"
+                    elif corners_dir is not None      and \
+                         context['f'][0] != '/'       and \
+                         not os.path.exists(context['f']):
+                        filename_canonical = f"{corners_dir}/{context['f']}"
                     else:
-                        filename_canonical = os.path.join(corners_dir, context['f'])
+                        filename_canonical = os.path.normpath(context['f'])
+
                     if accum_files(filename_canonical):
                         mapping[filename_canonical] = context['grid'].reshape(H,W,3)
 
@@ -522,8 +563,8 @@ pinhole pixel observations. Like opengv
             (np.isfinite(observation_qxqy_pinhole[..., 0])) * \
             (np.isfinite(observation_qxqy_pinhole[..., 1]))
 
-        if np.count_nonzero(i) < 4:
-            raise SolvePnPerror_toofew(f"Insufficient observations; need at least 4; got {np.count_nonzero(i)} instead. Cannot estimate initial extrinsics for {what}")
+        if np.count_nonzero(i) < 6:
+            raise SolvePnPerror_toofew(f"Insufficient observations; need at least 6; got {np.count_nonzero(i)} instead. Cannot estimate initial extrinsics for {what}")
 
         observations_local = observation_qxqy_pinhole[i]
         ref_object         = points_ref[i]
@@ -581,7 +622,10 @@ pinhole pixel observations. Like opengv
 def estimate_monocular_calobject_poses_Rt_tocam( indices_frame_camera,
                                                  observations,
                                                  object_spacing,
-                                                 models_or_intrinsics):
+                                                 models_or_intrinsics,
+                                                 *,
+                                                 # for diagnostics only
+                                                 paths = None):
     r"""Estimate camera-referenced poses of the calibration object from monocular views
 
 SYNOPSIS
@@ -592,7 +636,7 @@ SYNOPSIS
 
     print( observations.shape )
     ===>
-    (123, 3)
+    (123, 10,9, 3)
 
     models = [mrcal.cameramodel(f) for f in ("cam0.cameramodel",
                                              "cam1.cameramodel")]
@@ -628,7 +672,7 @@ SYNOPSIS
     # The reprojection error, comparing these hypothesis pixel observations from
     # what we actually observed. We estimated the calibration object pose from
     # the observations, so this should be small
-    err = q - observations[i_observation][:2]
+    err = q - observations[i_observation][...,:2]
 
     print( np.linalg.norm(err) )
     ===>
@@ -668,6 +712,9 @@ ARGUMENTS
   object_height_n,object_width_n arguments, but here we get those from the shape
   of the observations array
 
+- paths: optional iterable of strings, with the image filename corresponding to
+  each slice of observations. Used for diagnostics only
+
 - models_or_intrinsics: either
 
   - a list of mrcal.cameramodel objects from which we use the intrinsics
@@ -688,7 +735,7 @@ camera coordinate system FROM the calibration object coordinate system.
 
     lensmodels_intrinsics_data = [ m.intrinsics() if isinstance(m,mrcal.cameramodel) else m for m in models_or_intrinsics ]
     lensmodels                 = [di[0] for di in lensmodels_intrinsics_data]
-    intrinsics_data            = np.array([di[1] for di in lensmodels_intrinsics_data])
+    intrinsics_data            = [di[1] for di in lensmodels_intrinsics_data]
 
     if not all([mrcal.lensmodel_metadata_and_config(m)['has_core'] for m in lensmodels]):
         raise Exception("this currently works only with models that have an fxfycxcy core. It might not be required. Take a look at the following code if you want to add support")
@@ -719,11 +766,12 @@ camera coordinate system FROM the calibration object coordinate system.
 
         icam_intrinsics = indices_frame_camera[i_observation,1]
 
+        path_annotation = '' if paths is None else f'; "{paths[i_observation]}"'
         kwargs = dict( lensmodel         = lensmodels     [icam_intrinsics],
                        intrinsics_data   = intrinsics_data[icam_intrinsics],
                        observation_qxqyw = observations[i_observation],
                        points_ref        = points_ref,
-                       what              = f"observation {i_observation} (camera {icam_intrinsics})" )
+                       what              = f"observation {i_observation} (camera {icam_intrinsics}{path_annotation})" )
 
         Rt_cam_points = _estimate_camera_pose_from_fixed_point_observations(**kwargs)
         Rt_cam_points_all[i_observation, :, :] = Rt_cam_points
@@ -874,7 +922,10 @@ camera coordinate system FROM the points coordinate system.
     return Rt_cam_points_all
 
 
-def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
+def _estimate_camera_poses( # shape (Nobservations,4,3)
+                            calobject_poses_local_Rt_cf,
+                            # shape (Nobservations,2)
+                            indices_frame_camera, \
                             object_width_n, object_height_n,
                             object_spacing):
     r'''Estimate camera poses in respect to each other
@@ -893,23 +944,19 @@ def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
     indices_frame_camintrinsics_camextrinsics, which mrcal.optimize() expects
     '''
 
-    import heapq
-
     Ncameras = np.max(indices_frame_camera[:,1]) + 1
 
     # I need to compute an estimate of the pose of each camera in the coordinate
     # system of camera0. This is only possible if there're enough overlapping
     # observations. For instance if camera1 has overlapping observations with
-    # camera2, but neight overlap with camera0, then I can't relate camera1,2 to
-    # camera0. However if camera2 has overlap with camera2, then I can compute
-    # the relative pose of camera2 from its overlapping observations with
-    # camera0. And I can compute the camera1-camera2 pose from its overlapping
-    # data, and then transform to the camera0 coord system using the
-    # previously-computed camera2-camera0 pose
+    # camera2, but neither overlap with camera0, then I can't relate camera1,2 to
+    # camera0. However if camera0 has overlap with camera1, then I can compute
+    # the relative pose of camera2-camera0 from its overlapping observations with
+    # camera1.
     #
-    # I do this by solving a shortest-path problem using Dijkstra's algorithm to
-    # find a set of pair overlaps between cameras that leads to camera0. I favor
-    # edges with large numbers of shared observed frames
+    # I solve a shortest-path problem using Dijkstra's algorithm to find a set
+    # of pair overlaps between cameras that leads to camera0. I favor edges with
+    # large numbers of shared observed frames
 
     # list of camera-i to camera-0 transforms. I keep doing stuff until this
     # list is full of valid data
@@ -984,7 +1031,6 @@ def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
 
         return mrcal.align_procrustes_points_Rt01(A, B)
 
-
     def compute_connectivity_matrix():
         r'''Returns a connectivity matrix of camera observations
 
@@ -994,7 +1040,7 @@ def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
 
         '''
 
-        camera_connectivity = np.zeros( (Ncameras,Ncameras), dtype=int )
+        camera_connectivity = np.zeros( (Ncameras,Ncameras), dtype=np.uint16 )
         def finish_frame(i0, i1):
             for ic0 in range(i0, i1):
                 for ic1 in range(ic0+1, i1+1):
@@ -1017,86 +1063,33 @@ def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
         finish_frame(i_start_current, len(indices_frame_camera)-1)
         return camera_connectivity
 
+    def found_best_path_to_node(camera_idx, from_idx):
+        '''A shortest path was found'''
+        if camera_idx == 0:
+            # This is the reference camera. Nothing to do
+            return
 
+        Rt_fc = compute_pairwise_Rt(from_idx, camera_idx)
+
+        if from_idx == 0:
+            Rt_0c[camera_idx-1] = Rt_fc
+            return
+
+        Rt_0f = Rt_0c[from_idx-1]
+        Rt_0c[camera_idx-1] = mrcal.compose_Rt( Rt_0f, Rt_fc)
+
+    # shape (Ncamera,Ncameras)
+    # shape (Ncameras,Ncameras); each element is the number of shared
+    # observations
     shared_frames = compute_connectivity_matrix()
 
-    class Node:
-        def __init__(self, camera_idx):
-            self.camera_idx    = camera_idx
-            self.from_idx      = -1
-            self.cost_to_node  = None
+    # We need at least 2 shared observations to be useful; otherwise
+    # align_point_clouds() complains about insufficient overlap. I thus set to 0
+    # and too-sparse links, to make the graph traversal ignore those
+    shared_frames[shared_frames<2] = 0
 
-        def __lt__(self, other):
-            return self.cost_to_node < other.cost_to_node
-
-        def visit(self):
-            '''Dijkstra's algorithm'''
-            self.finish()
-
-            for neighbor_idx in range(Ncameras):
-                if neighbor_idx == self.camera_idx                  or \
-                   shared_frames[neighbor_idx,self.camera_idx] == 0:
-                    continue
-                neighbor = nodes[neighbor_idx]
-
-                if neighbor.visited():
-                    continue
-
-                cost_edge = Node.compute_edge_cost(shared_frames[neighbor_idx,self.camera_idx])
-
-                cost_to_neighbor_via_node = self.cost_to_node + cost_edge
-                if not neighbor.seen():
-                    neighbor.cost_to_node = cost_to_neighbor_via_node
-                    neighbor.from_idx     = self.camera_idx
-                    heapq.heappush(heap, neighbor)
-                else:
-                    if cost_to_neighbor_via_node < neighbor.cost_to_node:
-                        neighbor.cost_to_node = cost_to_neighbor_via_node
-                        neighbor.from_idx     = self.camera_idx
-                        heapq.heapify(heap) # is this the most efficient "update" call?
-
-        def finish(self):
-            '''A shortest path was found'''
-            if self.camera_idx == 0:
-                # This is the reference camera. Nothing to do
-                return
-
-            Rt_fc = compute_pairwise_Rt(self.from_idx, self.camera_idx)
-
-            if self.from_idx == 0:
-                Rt_0c[self.camera_idx-1] = Rt_fc
-                return
-
-            Rt_0f = Rt_0c[self.from_idx-1]
-            Rt_0c[self.camera_idx-1] = mrcal.compose_Rt( Rt_0f, Rt_fc)
-
-        def visited(self):
-            '''Returns True if this node went through the heap and has then been visited'''
-            return self.camera_idx == 0 or Rt_0c[self.camera_idx-1] is not None
-
-        def seen(self):
-            '''Returns True if this node has been in the heap'''
-            return self.cost_to_node is not None
-
-        @staticmethod
-        def compute_edge_cost(shared_frames):
-            # I want to MINIMIZE cost, so I MAXIMIZE the shared frames count and
-            # MINIMIZE the hop count. Furthermore, I really want to minimize the
-            # number of hops, so that's worth many shared frames.
-            cost = 100000 - shared_frames
-            assert(cost > 0) # dijkstra's algorithm requires this to be true
-            return cost
-
-
-
-    nodes = [Node(i) for i in range(Ncameras)]
-    nodes[0].cost_to_node = 0
-    heap = []
-
-    nodes[0].visit()
-    while heap:
-        node_top = heapq.heappop(heap)
-        node_top.visit()
+    mrcal.traverse_sensor_links( connectivity_matrix  = shared_frames,
+                                 callback_sensor_link = found_best_path_to_node )
 
     if any([x is None for x in Rt_0c]):
         raise Exception("ERROR: Don't have complete camera observations overlap!\n" +
@@ -1107,8 +1100,91 @@ def _estimate_camera_poses( calobject_poses_local_Rt_cf, indices_frame_camera, \
     return np.ascontiguousarray(nps.cat(*Rt_0c))
 
 
+def _traverse_sensor_links_python( Nsensors,
+                                   callback__neighbors,
+                                   callback__cost_edge,
+                                   callback__sensor_link ):
+    '''Traverses a connectivity graph of sensors
+
+    Starts from the root sensor (defined to have idx==0), and visits each one in
+    order of total distance from the root. Useful to evaluate the whole set of
+    sensors using pairwise metrics, building the network up from the
+    best-connected, to the worst-connected. Any sensor not connected to the root
+    at all will NOT be visited. The caller should check for any unvisited
+    sensors.
+
+    We have Nsensors sensors. Each one is identified by an integer in
+    [0,Nsensors). The root is defined to be sensor 0.
+
+    Three callbacks must be passed in:
+
+    - callback__neighbors (i)
+      An iterable returning each neigbor of sensor i.
+
+    - callback__cost_edge(i, i_parent)
+      The cost between two adjacent nodes
+
+    - callback__sensor_link(i, i_parent)
+      Called when the best path to node i is found. This path runs through
+      i_parent as the previous sensor
+
+    '''
+
+    import heapq
+
+    class Node:
+        def __init__(self, idx):
+            self.idx        = idx
+            self.idx_parent = -1
+            self.cost       = None
+            self.done       = False
+
+        def __lt__(self, other):
+            return self.cost < other.cost
+
+        def visit(self):
+            if self.idx != 0:
+                # We're not at the start; propagate the connection
+                callback__sensor_link(self.idx,
+                                      self.idx_parent)
+            self.done = True
+
+            for neighbor_idx in callback__neighbors(self.idx):
+                neighbor = nodes[neighbor_idx]
+
+                if neighbor.done:
+                    continue
+
+                cost_to_neighbor_via_node = \
+                    self.cost + \
+                    callback__cost_edge(neighbor_idx,self.idx)
+
+                if neighbor.cost is None:
+                    # Haven't seen this node yet
+                    neighbor.cost = cost_to_neighbor_via_node
+                    neighbor.idx_parent     = self.idx
+                    heapq.heappush(heap, neighbor)
+                else:
+                    # This node is already in the heap, ready to be processed.
+                    # If this new path to this node is better, use it
+                    if cost_to_neighbor_via_node < neighbor.cost:
+                        neighbor.cost = cost_to_neighbor_via_node
+                        neighbor.idx_parent     = self.idx
+                        heapq.heapify(heap) # is this the most efficient "update" call?
+
+
+    nodes = [Node(i) for i in range(Nsensors)]
+    nodes[0].cost = 0
+    heap = []
+
+    nodes[0].visit()
+    while heap:
+        node_top = heapq.heappop(heap)
+        node_top.visit()
+
+
 def estimate_joint_frame_poses(calobject_Rt_camera_frame,
-                               extrinsics_Rt_fromref,
+                               Rt_cam_ref,
                                indices_frame_camera,
                                object_width_n, object_height_n,
                                object_spacing):
@@ -1121,7 +1197,7 @@ SYNOPSIS
     ===>
     (123, 4,3)
 
-    print( extrinsics_Rt_fromref.shape )
+    print( Rt_cam_ref.shape )
     ===>
     (2, 4,3)
     # We have 3 cameras. The first one is at the reference coordinate system,
@@ -1131,14 +1207,14 @@ SYNOPSIS
     ===>
     (123, 2)
 
-    frames_rt_toref = \
+    rt_ref_frame = \
         mrcal.estimate_joint_frame_poses(calobject_Rt_camera_frame,
-                                         extrinsics_Rt_fromref,
+                                         Rt_cam_ref,
                                          indices_frame_camera,
                                          object_width_n, object_height_n,
                                          object_spacing)
 
-    print( frames_rt_toref.shape )
+    print( rt_ref_frame.shape )
     ===>
     (87, 6)
 
@@ -1156,13 +1232,13 @@ SYNOPSIS
 
     # The estimated calibration object points in the reference coordinate
     # system, for this one observation
-    pref = mrcal.transform_point_rt( frames_rt_toref[iframe],
+    pref = mrcal.transform_point_rt( rt_ref_frame[iframe],
                                      calobject )
 
     # The estimated calibration object points in the camera coord system. Camera
     # 0 is at the reference
     if icam >= 1:
-        pcam = mrcal.transform_point_Rt( extrinsics_Rt_fromref[icam-1],
+        pcam = mrcal.transform_point_Rt( Rt_cam_ref[icam-1],
                                          pref )
     else:
         pcam = pref
@@ -1172,7 +1248,7 @@ SYNOPSIS
 
     # The reprojection error, comparing these hypothesis pixel observations from
     # what we actually observed. This should be small
-    err = q - observations[i_observation][:2]
+    err = q - observations[i_observation][...,:2]
 
     print( np.linalg.norm(err) )
     ===>
@@ -1195,7 +1271,7 @@ observing-camera coordinate system poses we already have.
 By convention, we have a "reference" coordinate system that ties the poses of
 all the frames (calibration objects) and the cameras together. And by
 convention, this "reference" coordinate system is the coordinate system of
-camera 0. Thus the array of camera poses extrinsics_Rt_fromref holds Ncameras-1
+camera 0. Thus the array of camera poses Rt_cam_ref holds Ncameras-1
 transformations: the first camera has an identity transformation, by definition.
 
 This function assumes we're observing a moving object from stationary cameras
@@ -1211,7 +1287,7 @@ ARGUMENTS
   calibration object coordinate system. This is returned by
   estimate_monocular_calobject_poses_Rt_tocam()
 
-- extrinsics_Rt_fromref: an array of shape (Ncameras-1,4,3). Each slice is an Rt
+- Rt_cam_ref: an array of shape (Ncameras-1,4,3). Each slice is an Rt
   transformation TO the camera coordinate system FROM the reference coordinate
   system. By convention camera 0 defines the reference coordinate system, so
   that camera's extrinsics are the identity, by definition, and we don't store
@@ -1237,7 +1313,7 @@ system FROM the calibration object coordinate system.
 
     '''
 
-    Rt_ref_cam = mrcal.invert_Rt( extrinsics_Rt_fromref )
+    Rt_ref_cam = mrcal.invert_Rt( Rt_cam_ref )
 
 
     def Rt_ref_frame(i_observation0, i_observation1):
@@ -1290,7 +1366,7 @@ system FROM the calibration object coordinate system.
 
 
 
-    frames_rt_toref = np.array(())
+    rt_ref_frame = np.array(())
 
     iframe_current          = -1
     i_observation_framestart = -1;
@@ -1302,9 +1378,9 @@ system FROM the calibration object coordinate system.
             if i_observation_framestart >= 0:
                 Rt = Rt_ref_frame(i_observation_framestart,
                                   i_observation)
-                frames_rt_toref = nps.glue(frames_rt_toref,
-                                           mrcal.rt_from_Rt(Rt),
-                                           axis=-2)
+                rt_ref_frame = nps.glue(rt_ref_frame,
+                                        mrcal.rt_from_Rt(Rt),
+                                        axis=-2)
 
             i_observation_framestart = i_observation
             iframe_current          = iframe
@@ -1312,18 +1388,21 @@ system FROM the calibration object coordinate system.
     if i_observation_framestart >= 0:
         Rt = Rt_ref_frame(i_observation_framestart,
                           indices_frame_camera.shape[0])
-        frames_rt_toref = nps.glue(frames_rt_toref,
-                                   mrcal.rt_from_Rt(Rt),
-                                   axis=-2)
+        rt_ref_frame = nps.glue(rt_ref_frame,
+                                mrcal.rt_from_Rt(Rt),
+                                axis=-2)
 
-    return frames_rt_toref
+    return rt_ref_frame
 
 
 def seed_stereographic( imagersizes,
                         focal_estimate,
                         indices_frame_camera,
                         observations,
-                        object_spacing):
+                        object_spacing,
+                        *,
+                        # for diagnostics only
+                        paths = None):
     r'''Compute an optimization seed for a camera calibration
 
 SYNOPSIS
@@ -1341,8 +1420,8 @@ SYNOPSIS
     (123, 3)
 
     intrinsics_data,       \
-    extrinsics_rt_fromref, \
-    frames_rt_toref =      \
+    rt_cam_ref, \
+    rt_ref_frame =      \
         mrcal.seed_stereographic(imagersizes          = imagersizes,
                                  focal_estimate       = 1500,
                                  indices_frame_camera = indices_frame_camera,
@@ -1352,8 +1431,8 @@ SYNOPSIS
     ....
 
     mrcal.optimize(intrinsics            = intrinsics_data,
-                   extrinsics_rt_fromref = extrinsics_rt_fromref,
-                   frames_rt_toref       = frames_rt_toref,
+                   rt_cam_ref            = rt_cam_ref,
+                   rt_ref_frame          = rt_ref_frame,
                    lensmodel             = 'LENSMODEL_STEREOGRAPHIC',
                    ...)
 
@@ -1373,7 +1452,7 @@ to invoking the optimizer.
 By convention, we have a "reference" coordinate system that ties the poses of
 all the frames (calibration objects) and the cameras together. And by
 convention, this "reference" coordinate system is the coordinate system of
-camera 0. Thus the array of camera poses extrinsics_rt_fromref holds Ncameras-1
+camera 0. Thus the array of camera poses rt_cam_ref holds Ncameras-1
 transformations: the first camera has an identity transformation, by definition.
 
 This function assumes we're observing a moving object from stationary cameras
@@ -1416,6 +1495,9 @@ ARGUMENTS
   object_height_n,object_width_n arguments, but here we get those from the shape
   of the observations array
 
+- paths: optional iterable of strings, with the image filename corresponding to
+  each slice of observations. Used for diagnostics only
+
 RETURNED VALUES
 
 We return a tuple:
@@ -1430,13 +1512,13 @@ We return a tuple:
   the proper number of parameters to match whatever lens model we're using, and
   then invoke the optimizer.
 
-- extrinsics_rt_fromref: an array of shape (Ncameras-1,6). Each slice is an rt
+- rt_cam_ref: an array of shape (Ncameras-1,6). Each slice is an rt
   transformation TO the camera coordinate system FROM the reference coordinate
   system. By convention camera 0 defines the reference coordinate system, so
   that camera's extrinsics are the identity, by definition, and we don't store
   that data in this array
 
-- frames_rt_toref: an array of shape (Nframes,6). Each slice represents the pose
+- rt_ref_frame: an array of shape (Nframes,6). Each slice represents the pose
   of the calibration object at one instant in time: an rt transformation TO the
   reference coordinate system FROM the calibration object coordinate system.
 
@@ -1481,7 +1563,8 @@ We return a tuple:
         mrcal.estimate_monocular_calobject_poses_Rt_tocam( indices_frame_camera,
                                                            observations,
                                                            object_spacing,
-                                                           intrinsics)
+                                                           intrinsics,
+                                                           paths = paths)
     # these map FROM the coord system of the calibration object TO the coord
     # system of this camera
 
@@ -1505,24 +1588,24 @@ We return a tuple:
     if len(camera_poses_Rt_0_cami):
         # extrinsics should map FROM the ref coord system TO the coord system of the
         # camera in question. This is backwards from what I have
-        extrinsics_Rt_fromref = \
+        Rt_cam_ref = \
             nps.atleast_dims( mrcal.invert_Rt(camera_poses_Rt_0_cami),
                               -3 )
     else:
-        extrinsics_Rt_fromref = np.zeros((0,4,3))
+        Rt_cam_ref = np.zeros((0,4,3))
 
-    frames_rt_toref = \
+    rt_ref_frame = \
         mrcal.estimate_joint_frame_poses(
             calobject_poses_local_Rt_cf,
-            extrinsics_Rt_fromref,
+            Rt_cam_ref,
             indices_frame_camera,
             object_width_n, object_height_n,
             object_spacing)
 
     return \
         nps.cat(*[i[1] for i in intrinsics]), \
-        nps.atleast_dims(mrcal.rt_from_Rt(extrinsics_Rt_fromref), -2), \
-        frames_rt_toref
+        nps.atleast_dims(mrcal.rt_from_Rt(Rt_cam_ref), -2), \
+        rt_ref_frame
 
 
 def _compute_valid_intrinsics_region(model,
@@ -1755,10 +1838,10 @@ This function returns a tuple
     residuals_shape = observations.shape[:-1] + (2,)
 
     # shape (Nobservations,object_height_n,object_width_n,2)
-    residuals = \
+    x = \
         mrcal.optimizer_callback(**optimization_inputs,
                                  no_jacobian      = True,
-                                 no_factorization = True)[1][:np.product(residuals_shape)]. \
+                                 no_factorization = True)[1][:np.prod(residuals_shape)]. \
         reshape(*residuals_shape)
 
     # shape (Nobservations, object_height_n, object_width_n)
@@ -1770,7 +1853,7 @@ This function returns a tuple
     idx[ observations[...,2] <= 0.0 ] = False
 
     # shape (N,2)
-    err = residuals   [idx, ...    ]
+    err = x           [idx, ...    ]
     obs = observations[idx, ..., :2]
 
     # Each has shape (Nheight,Nwidth)
