@@ -17,38 +17,46 @@ int Ntests       = 0;
 int NtestsFailed = 0;
 
 __attribute__((unused))
-static void _confirm(bool x, const char* what_x, int where)
+static bool _confirm(bool x, const char* what_x, int where)
 {
     Ntests++;
 
     if(x)
+    {
         printf(GREEN "OK: %s is true" COLOR_RESET"\n",
                what_x);
+        return true;
+    }
     else
     {
         printf(RED "FAIL on line %d: %s is false" COLOR_RESET"\n", where, what_x);
         NtestsFailed++;
+        return false;
     }
 }
 #define confirm(x) _confirm(x, #x, __LINE__)
 
 __attribute__((unused))
-static void _confirm_eq_int(int x, int xref, const char* what_x, const char* what_xref, int where)
+static bool _confirm_eq_int(int x, int xref, const char* what_x, const char* what_xref, int where)
 {
     Ntests++;
     if(x == xref)
+    {
         printf(GREEN "OK: %s == %s, as it should" COLOR_RESET"\n", what_x, what_xref);
+        return true;
+    }
     else
     {
         printf(RED "FAIL on line %d: %s != %s: %d != %d" COLOR_RESET"\n",
                where, what_x, what_xref, x, xref);
         NtestsFailed++;
+        return false;
     }
 }
 #define confirm_eq_int(x,xref) _confirm_eq_int(x, xref, #x, #xref, __LINE__)
 
 __attribute__((unused))
-static void _confirm_eq_int_max_array(const int* x, const int* xref, int N, const char* what_x, const char* what_xref, int where)
+static bool _confirm_eq_int_max_array(const int* x, const int* xref, int N, const char* what_x, const char* what_xref, int where)
 {
     Ntests++;
 
@@ -58,10 +66,11 @@ static void _confirm_eq_int_max_array(const int* x, const int* xref, int N, cons
             printf(RED "FAIL on line %d: %s != %s" COLOR_RESET"\n",
                    where, what_x, what_xref);
             NtestsFailed++;
-            return;
+            return false;
         }
 
     printf(GREEN "OK: %s = %s, as it should" COLOR_RESET"\n", what_x, what_xref);
+    return true;
 }
 #define confirm_eq_int_max_array(x,xref, N)                      \
     _confirm_eq_int_max_array(x, xref, N, #x, #xref, __LINE__)
@@ -83,7 +92,7 @@ static void _confirm_eq_double(double x, double xref, const char* what_x, const
     _confirm_eq_double(x, xref, #x, #xref, eps, __LINE__)
 
 __attribute__((unused))
-static void _confirm_eq_double_max_array(const double* x, const double* xref, int N, const char* what_x, const char* what_xref, double eps, int where)
+static bool _confirm_eq_double_max_array(const double* x, const double* xref, int N, const char* what_x, const char* what_xref, double eps, int where)
 {
     double worst_err = 0.0;
     for(int i=0; i<N; i++)
@@ -93,12 +102,16 @@ static void _confirm_eq_double_max_array(const double* x, const double* xref, in
     }
     Ntests++;
     if(worst_err < eps)
+    {
         printf(GREEN "OK: %s ~ %s, as it should" COLOR_RESET"\n", what_x, what_xref);
+        return true;
+    }
     else
     {
         printf(RED "FAIL on line %d: %s !~ %s" COLOR_RESET"\n",
                where, what_x, what_xref);
         NtestsFailed++;
+        return false;
     }
 }
 #define confirm_eq_double_max_array(x,xref, N, eps)                      \
