@@ -28,6 +28,42 @@ for lensmodel in ('LENSMODEL_LATLON', 'LENSMODEL_PINHOLE'):
 
     az_fov_deg = 90
     el_fov_deg = 50
+
+
+    # made-up numbers. I'm just making sure rectified_resolution() does what it
+    # is supposed to
+    pixels_per_deg_azel_ref = \
+        mrcal.stereo._rectified_resolution_python( model0,
+                                                   az_fov_deg = az_fov_deg,
+                                                   el_fov_deg = el_fov_deg,
+                                                   az0_deg = -4.,
+                                                   el0_deg = 6.,
+                                                   R_cam0_rect0 = mrcal.identity_R(),
+                                                   pixels_per_deg_az = -1./8.,
+                                                   pixels_per_deg_el = -1./4.,
+                                                   rectification_model = lensmodel)
+    pixels_per_deg_azel_computed = \
+        mrcal.rectified_resolution( model0,
+                                    az_fov_deg = az_fov_deg,
+                                    el_fov_deg = el_fov_deg,
+                                    az0_deg = -4.,
+                                    el0_deg = 6.,
+                                    R_cam0_rect0 = mrcal.identity_R(),
+                                    pixels_per_deg_az = -1./8.,
+                                    pixels_per_deg_el = -1./4.,
+                                    rectification_model = lensmodel)
+    testutils.confirm_equal(pixels_per_deg_azel_computed,
+                            pixels_per_deg_azel_ref,
+                            msg=f'computed resolution matches between the C and Python implementations ({lensmodel})')
+
+
+    models_rectified_ref = \
+        mrcal.stereo._rectified_system_python( (model0, model1),
+                                               az_fov_deg = az_fov_deg,
+                                               el_fov_deg = el_fov_deg,
+                                               pixels_per_deg_az = -1./8.,
+                                               pixels_per_deg_el = -1./4.,
+                                               rectification_model = lensmodel)
     models_rectified = \
         mrcal.rectified_system( (model0, model1),
                                 az_fov_deg = az_fov_deg,
@@ -35,6 +71,25 @@ for lensmodel in ('LENSMODEL_LATLON', 'LENSMODEL_PINHOLE'):
                                 pixels_per_deg_az = -1./8.,
                                 pixels_per_deg_el = -1./4.,
                                 rectification_model = lensmodel)
+    for icam in (0,1):
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[0],
+                                models_rectified_ref[icam].intrinsics()[0],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching lensmodel between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[1],
+                                models_rectified_ref[icam].intrinsics()[1],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching intrinsics between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].rt_ref_cam(),
+                                models_rectified_ref[icam].rt_ref_cam(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching extrinsics between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].imagersize(),
+                                models_rectified_ref[icam].imagersize(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching imagersize between the C and Python implementations ({lensmodel})')
+
+
     az0 = 0.
     el0 = 0.
 
@@ -136,7 +191,44 @@ for lensmodel in ('LENSMODEL_LATLON', 'LENSMODEL_PINHOLE'):
     rt01 = np.array((0.1, 0.2, 0.05,  3.0, 0.2, 1.0))
     model1.rt_ref_cam( mrcal.compose_rt(model0.rt_ref_cam(),
                                         rt01))
+
+
+    # made-up numbers. I'm just making sure rectified_resolution() does what it
+    # is supposed to
+    pixels_per_deg_azel_ref = \
+        mrcal.stereo._rectified_resolution_python( model0,
+                                                   az_fov_deg = az_fov_deg,
+                                                   el_fov_deg = el_fov_deg,
+                                                   az0_deg = 4.,
+                                                   el0_deg = 6.,
+                                                   R_cam0_rect0 = mrcal.identity_R(),
+                                                   pixels_per_deg_az = -1./8.,
+                                                   pixels_per_deg_el = -1./4.,
+                                                   rectification_model = lensmodel)
+    pixels_per_deg_azel_computed = \
+        mrcal.rectified_resolution( model0,
+                                    az_fov_deg = az_fov_deg,
+                                    el_fov_deg = el_fov_deg,
+                                    az0_deg = 4.,
+                                    el0_deg = 6.,
+                                    R_cam0_rect0 = mrcal.identity_R(),
+                                    pixels_per_deg_az = -1./8.,
+                                    pixels_per_deg_el = -1./4.,
+                                    rectification_model = lensmodel)
+    testutils.confirm_equal(pixels_per_deg_azel_computed,
+                            pixels_per_deg_azel_ref,
+                            msg=f'computed resolution matches between the C and Python implementations ({lensmodel})')
+
+
     el0_deg = 10.0
+    models_rectified_ref = \
+        mrcal.stereo._rectified_system_python( (model0, model1),
+                                               az_fov_deg = az_fov_deg,
+                                               el_fov_deg = el_fov_deg,
+                                               el0_deg    = el0_deg,
+                                               pixels_per_deg_az = -1./8.,
+                                               pixels_per_deg_el = -1./4.,
+                                               rectification_model = lensmodel)
     models_rectified = \
         mrcal.rectified_system( (model0, model1),
                                 az_fov_deg = az_fov_deg,
@@ -145,6 +237,27 @@ for lensmodel in ('LENSMODEL_LATLON', 'LENSMODEL_PINHOLE'):
                                 pixels_per_deg_az = -1./8.,
                                 pixels_per_deg_el = -1./4.,
                                 rectification_model = lensmodel)
+    for icam in (0,1):
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[0],
+                                models_rectified_ref[icam].intrinsics()[0],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching lensmodel between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[1],
+                                models_rectified_ref[icam].intrinsics()[1],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching intrinsics between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].rt_ref_cam(),
+                                models_rectified_ref[icam].rt_ref_cam(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching extrinsics between the C and Python implementations ({lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].imagersize(),
+                                models_rectified_ref[icam].imagersize(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching imagersize between the C and Python implementations ({lensmodel})')
+
+
+
+
     el0 = el0_deg*np.pi/180.
     # az0 is the "forward" direction of the two cameras, relative to the
     # baseline vector
@@ -394,4 +507,48 @@ for lensmodel in ('LENSMODEL_LATLON', 'LENSMODEL_PINHOLE'):
                              msg=f'stereo_unproject (scalar) reports the right thing ({lensmodel})')
 
 
+
+
+    # check the python/c difference with a larger margin to make sure that this
+    # is handled identically
+    az_edge_margin_deg = 40.
+    models_rectified_ref = \
+        mrcal.stereo._rectified_system_python( (model0, model1),
+                                               az_fov_deg = az_fov_deg,
+                                               el_fov_deg = el_fov_deg,
+                                               el0_deg    = el0_deg,
+                                               pixels_per_deg_az = -1./8.,
+                                               pixels_per_deg_el = -1./4.,
+                                               az_edge_margin_deg = az_edge_margin_deg,
+                                               rectification_model = lensmodel)
+    models_rectified = \
+        mrcal.rectified_system( (model0, model1),
+                                az_fov_deg = az_fov_deg,
+                                el_fov_deg = el_fov_deg,
+                                el0_deg    = el0_deg,
+                                pixels_per_deg_az = -1./8.,
+                                pixels_per_deg_el = -1./4.,
+                                az_edge_margin_deg = az_edge_margin_deg,
+                                rectification_model = lensmodel)
+    for icam in (0,1):
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[0],
+                                models_rectified_ref[icam].intrinsics()[0],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching lensmodel between the C and Python implementations ({az_edge_margin_deg=} {lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].intrinsics()[1],
+                                models_rectified_ref[icam].intrinsics()[1],
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching intrinsics between the C and Python implementations ({az_edge_margin_deg=} {lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].rt_ref_cam(),
+                                models_rectified_ref[icam].rt_ref_cam(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching extrinsics between the C and Python implementations ({az_edge_margin_deg=} {lensmodel})')
+        testutils.confirm_equal(models_rectified    [icam].imagersize(),
+                                models_rectified_ref[icam].imagersize(),
+                                eps = 1e-5,
+                                msg=f'model{icam} has matching imagersize between the C and Python implementations ({az_edge_margin_deg=} {lensmodel})')
+
+
+
+
 testutils.finish()
